
using System;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using Microsoft.Win32;
using System.Collections.Generic;

namespace TCPIPClient
{
    public partial class Client : Form
    {
        TcpClient mTcpClient; byte[] mRx; static string strName; bool isAvailableToWrite = true;

        public Client()
        {
            InitializeComponent();
            btnClose.TabStop = false;
            btnClose.FlatAppearance.BorderSize = 0;
            btnClose.FlatStyle = FlatStyle.Flat;
            tbConsole.TabStop = false;
            tbName.TabIndex = 1;
        }
        void GetInstalledApps()
        {
            List<string> items = new List<string>();
            string SoftwareKey = @"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall";
            using (RegistryKey rk = Registry.LocalMachine.OpenSubKey(SoftwareKey))
            {
                foreach (string skName in rk.GetSubKeyNames())
                {
                    using (RegistryKey sk = rk.OpenSubKey(skName))
                    {
                        try
                        {

                            if (sk.GetValue("DisplayName") != null)
                            {
                                listBox1.Items.Add(sk.GetValue("DisplayName"));

                            }
                        }
                        catch (Exception ex)
                        {
                            MessageBox.Show(ex.Message, "Exception",
                                MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        }
                    }
                    items.Clear();
                }
            }

        }

        private void button2_Click(object sender, EventArgs e)
        {
            GetInstalledApps();
        }
        private void Client_Load(object sender, EventArgs e){ }

        IPAddress findmMyIPV4Address()
        {
            string strThisHostName = string.Empty;
            IPHostEntry thisHostDNSEntry = null;
            IPAddress[] allIPsOfThisHost = null;
            IPAddress ipv4Ret = null;

            try
            {
                strThisHostName = System.Net.Dns.GetHostName();
                
                thisHostDNSEntry = System.Net.Dns.GetHostEntry(strThisHostName);
                
                allIPsOfThisHost = thisHostDNSEntry.AddressList;

                for (int idx = allIPsOfThisHost.Length -1; idx > 0; idx --)
                {
                  
                    if (allIPsOfThisHost[idx].AddressFamily == AddressFamily.InterNetwork)
                    {
                        ipv4Ret = allIPsOfThisHost[idx];
                        break;
                    }
                }

            } catch (Exception exc)
            {
               
                MessageBox.Show("IP Address cannot be detected. Program will still carry on running...", "TCP/IP Client", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            return ipv4Ret;
        }

        protected override void WndProc(ref Message m)
        {
            base.WndProc(ref m);
            if (m.Msg == WM_NCHITTEST)
                m.Result = (IntPtr)(HT_CAPTION);
        }

        private const int WM_NCHITTEST = 0x84;
        private const int HT_CLIENT = 0x1;
        private const int HT_CAPTION = 0x2;
      
        private void btnConnect_Click(object sender, EventArgs e)
        {
          
            IPAddress ipa;
            int nPort;

           
            isAvailableToWrite = false;

            try
            {
                
                if (string.IsNullOrEmpty(tbServerIP.Text) || string.IsNullOrEmpty(tbName.Text))
                {
                    MessageBox.Show("You must enter your Name and provide an IP Address!", "TCP/IP Client", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                if (!IPAddress.TryParse(tbServerIP.Text, out ipa))
                {
                    MessageBox.Show("Please supply an IP Address!", "TCP/IP Client", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }
                if (!int.TryParse(tbServerPort.Text, out nPort))
                {
                    
                    nPort = 23000;
                    tbServerPort.Text = Convert.ToString(nPort);
                }

                if (!string.IsNullOrEmpty(tbName.Text))
                {
                    strName = tbName.Text;
                }

                
                mTcpClient = new TcpClient();
             
                mTcpClient.BeginConnect(ipa, nPort, onCompleteConnect, mTcpClient);

                checkConnection();

            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "TCP/IP Client", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }
        void checkConnection()
        {
            if (isAvailableToWrite)
            {
                connectionGroup.Enabled = true;
                msgControls.Enabled = false;
            }
            else if (!isAvailableToWrite)
            {
                connectionGroup.Enabled = false;
                msgControls.Enabled = true;
                tbPayLoad.Select();
            }
        }
        void onCompleteConnect(IAsyncResult iar)
        {
            TcpClient tcpc;
            IPAddress ipa = null;
            ipa = findmMyIPV4Address();
            isAvailableToWrite = false;
            byte[] tx = new byte[512];

            try
            {
                
                tcpc = (TcpClient)iar.AsyncState;
                tcpc.EndConnect(iar);
                mRx = new byte[512];
                tcpc.GetStream().BeginRead(mRx, 0, mRx.Length, onCompleteReadFromServerStream, tcpc);
            
                tx = Encoding.ASCII.GetBytes("Client has been connected. \r\nClient Name: " + strName + "\r\n"
                + "Clients IP Address: " + ipa.ToString() + "\r\n");
                mTcpClient.GetStream().BeginWrite(tx, 0, tx.Length, onCompleteWriteToServer, mTcpClient);
                

            } catch (Exception exc)
            {
                MessageBox.Show("The Server is offline or has actively refused your connection, please try again later!",
                    "TCP/IP Client", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Application.Restart();
               
            }
        }

       
        void onCompleteReadFromServerStream (IAsyncResult iar)
        {
            TcpClient tcpc;
            int nCountBytesReceivedFromServer;
            string strReceived;
            try
            {
                tcpc = (TcpClient)iar.AsyncState;
                nCountBytesReceivedFromServer = tcpc.GetStream().EndRead(iar);
                if (nCountBytesReceivedFromServer == 0)
                {
                    MessageBox.Show("Connection broken, retry connecting to the server!", "TCP/IP Client", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                strReceived = Encoding.ASCII.GetString(mRx, 0, nCountBytesReceivedFromServer);
                printLine(strReceived);
                mRx = new byte[512];
                tcpc.GetStream().BeginRead(mRx, 0, mRx.Length, onCompleteReadFromServerStream, tcpc);

            } catch (Exception exc)
            {
                MessageBox.Show("Server has disconnected, please try again later!", "TCP/IP Client" , MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                Application.Restart();
            }
        }
        private void btnSend_Click(object sender, EventArgs e)
        {
            byte[] tx;
            if (string.IsNullOrEmpty(tbPayLoad.Text))
            {
                MessageBox.Show("There is nothing to send as your message box is empty!", "TCP/IP Client", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }

           
            try
            {
                
                if (mTcpClient != null)
                {
                    if (mTcpClient.Client.Connected)
                    {
                      
                        tx = Encoding.ASCII.GetBytes(strName + DateTime.Now.ToString(" [h:mm:ss] ") + ": " + tbPayLoad.Text);
                        mTcpClient.GetStream().BeginWrite(tx, 0, tx.Length, onCompleteWriteToServer, mTcpClient);

                       
                        printLine(strName + DateTime.Now.ToString(" [h:mm:ss] ") + ": " + tbPayLoad.Text);

                      
                        tbPayLoad.Clear();

                    } else if (!mTcpClient.Client.Connected)
                    {
                        MessageBox.Show("You are not connected to a server!", "TCP/IP Client", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        msgControls.Enabled = false;
                    }
                } 

            } catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "TCP/IP Client", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

       
        void onCompleteWriteToServer (IAsyncResult iar)
        {
           
            TcpClient tcpc;

            try
            { 
               
                tcpc = (TcpClient)iar.AsyncState;
                tcpc.GetStream().EndWrite(iar);

            } catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "TCP/IP Client", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
       
        private void btnClose_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
       
        public void printLine(string _strPrint)
        {
            tbConsole.Invoke(new Action<string>(doInvoke), _strPrint);
        }

        public void doInvoke(string _strPrint)
        {
            tbConsole.Text = tbConsole.Text + _strPrint + Environment.NewLine;
        }

        private void Client_Load_1(object sender, EventArgs e)
        {

        }

        private void button3_Click(object sender, EventArgs e)
        {
            byte[] tx;

           
            if (string.IsNullOrEmpty(listBox1.SelectedItem.ToString()))
            {
                MessageBox.Show("There is nothing to send as your message box is empty!", "TCP/IP Client", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }
            try
            {
                if (mTcpClient != null)
                {
                    if (mTcpClient.Client.Connected)
                    {
                        
                        tx = Encoding.ASCII.GetBytes(strName + DateTime.Now.ToString(" [h:mm:ss] ") + ": " + listBox1.SelectedItem.ToString());
                        mTcpClient.GetStream().BeginWrite(tx, 0, tx.Length, onCompleteWriteToServer, mTcpClient);
                        printLine(strName + DateTime.Now.ToString(" [h:mm:ss] ") + ": " + listBox1.SelectedItem.ToString());
                        tbPayLoad.Clear();

                    }
                    else if (!mTcpClient.Client.Connected)
                    {
                        MessageBox.Show("You are not connected to a server!", "TCP/IP Client", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        msgControls.Enabled = false;
                    }
                }

            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message, "TCP/IP Client", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        
    }
}
